/************************************************************************
 *   IRC - Internet Relay Chat, include/cmodes.h
 *   This file is copyright (C) 2001 Andrew Suffield
 *                                    <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifndef INCLUDED_m_cmodes_h
#define INCLUDED_m_cmodes_h
#ifndef INCLUDED_config_h
#include "config.h"
#endif
#include "ircd_defs.h"
#include "umodes.h"
#include "numeric.h"

#include <sys/types.h>

struct Channel;
struct Client;

/* These are the different classes of channel mode.
 *
 * CMTYPE_CHANFLAG is modes such as +tnismclkPF, which are either "on" or "off"
 *                 for the channel
 * CMTYPE_USERFLAG is modes such as +ovh, which are "on" or "off" for a single user
 * CMTYPE_MASK is modes such as +bdeIq, which are masks attached to the channel
 * CMTYPE_INTVAL is a mode such as +l, which has an integer value
 * CMTYPE_STRVAL is modes such as +fk, which have string values
 */

#define CMTYPE_CHANFLAG 0x00000001
#define CMTYPE_USERFLAG 0x00000002
#define CMTYPE_MASK     0x00000004
#define CMTYPE_INTVAL   0x00000008
#define CMTYPE_STRVAL   0x00000010

struct channel_mode
{
  u_int32_t value;		/* This will be replaced by the index during init for strval, intval, and mask types */
  char letter;
  u_int32_t type;
  int umode_required;		/* UMODE_INVALID if any channel operator can set it */
  int position;
};

struct channel_mask
{
  char nick[NICKLEN + 1];
  char user[USERLEN + 1];
  char host[HOSTLEN + 1];
  char channel[CHANNELLEN + 1];
  time_t timeout;
  char reason[REASONLEN + 1];

  char placed_by[NICKLEN + 1];
  time_t placed_at;
};

#if 0				/* None of these are needed any more */
#define CHFL_CHANOP     0x00000001	/* Channel operator */
#define CHFL_VOICE      0x00000002	/* the power to speak */
/*#define CHFL_DEOPPED    0x00000004 *//* deopped by us, modes need to be bounced *//* now obsolete -- asuffield */
#define CHFL_BAN        0x00000008	/* ban channel flag */
#define CHFL_EXCEPTION  0x00000010	/* exception to ban channel flag */
#define CHFL_DENY       0x00000020	/* regular expression deny flag */
#define CHFL_INVEX      0x00000040	/* invex channel flag */
#define CHFL_QUIET      0x00000080	/* quieted on channel -- asuffield */
#endif

/* These are renamed because the code to use them has changed */
#define CUFLAG_CHANOP       0x00000001
#define CUFLAG_VOICE        0x00000002

#define CUFLAG_EXCEPTION    0x80000000	/* there's an exception on this user. used to delete exceptions on kick */

#define CMODE_SECRET        0x00000010
#define CMODE_MODERATED     0x00000020
#define CMODE_TOPICLIMIT    0x00000040
#define CMODE_INVITEONLY    0x00000080
#define CMODE_NOPRIVMSGS    0x00000100
#define CMODE_NOCOLOUR      0x00010000
#define CMODE_PERM          0x00020000
#define CMODE_ALLOWFORWARD  0x00040000
#define CMODE_JUPED         0x00080000

extern struct channel_mode cmodes[];

/* These affect the memory allocated for storing values in struct Channel,
 * so then need to be correct (will abort on load if too small)
 */
#define CMODE_INTVAL_COUNT 1
#define CMODE_STRVAL_COUNT 2
#define CMODE_MASK_COUNT 5

/* This is -1 for invalid modes */
extern int cmode_index[];	/* letter -> index in channel_modes */

extern void init_cmode_tables(void);
extern struct SLink *match_channel_mask(const struct channel_mask *,
					struct Channel *, char mode);
extern void free_channel_modes(struct Channel *);
extern void delete_channel_mask(struct Channel *, struct SLink *, u_int32_t);
extern void channel_modes(const struct Client *, char *, char *,
			  const struct Channel *, int);
extern void change_chan_flag(struct Channel *, struct Client *, int, int);
extern int parse_mode_char(struct Client *, struct Channel *, char, char,
			   int, char **, char **, char **, char **, int *,
			   int *, char ***);
extern void collapse_signs(char *);
extern int parse_channel_mask(const char *, struct channel_mask *, int *,
			      char ***);
extern char *build_channel_mask(const struct channel_mask *);
extern struct SLink *find_channel_mask(const struct channel_mask *,
				       struct Channel *, u_int32_t, int, int);
extern struct channel_mask *build_mask(const struct Client *);
extern void set_channel_mode(struct Client *, struct Client *,
			     struct Channel *, int, char **);
extern int get_mode_intval(struct Channel *, char);
extern char *get_mode_strval(struct Channel *, char);

#endif
